#!/usr/bin/env python
	
# Authors:Xiaohong Zhuang, Dr.Jeffery B. Klauda
# Date:10/5/2015
	
# This script calculate the total edp of system and cut the edge of lipid bilayer
import numpy as np

# Read in lipid names
filetoread = "lip_all.txt" 
fob=open(filetoread,'r')
lips=fob.readlines()
lip_list = map(lambda s: s.strip(), lips) 

# Calculate the total edp of system
# Read total edp of each lipid
for k in range(0,len(lip_list),1):
	lip=lip_list[k];
	filetoread = "%s-edp.txt" % lip
	EDP=np.genfromtxt(filetoread,delimiter="    ")
	z=EDP[:,0];D=EDP[:,1];
	zr=z.reshape(-1,1);Dr=D.reshape(-1,1)
	if k==0:
		zD=np.concatenate((zr,Dr),axis=1)
	else:
		zD=np.concatenate((zD,Dr),axis=1)

Dtot_Row=np.sum(zD[:,1:],axis=1)
Dtotr=Dtot_Row.reshape(-1,1)
zDtot=np.concatenate((zr,Dtotr),axis=1)	

filetowrite = "tot_edp.txt"
np.savetxt(filetowrite,zDtot,fmt='%-2.5f',delimiter='    ')

# CUT THE EDGE
# Read number of water per lipid
fob = open('num_wat.txt','r')
N_wat = fob.readline()
N_wat = N_wat.strip()
N_wat = int(float(N_wat))
fob.close()

# data is an array contains z and density column, bd_s= set boundary slope, w=width of plateau
def plateau(data,dz,bd_s,N_wat):
	if 'pi' in lip:
		w=N_wat/5.0-4.0;
	else:
		w=N_wat/5.0-3.0
	j=int(w/(dz*2)); N=len(data[:,0])-1;
	for i in range(j+1,N/2-1,1):
		dD1=data[i-j+1,1]-data[i-j,1];dD2=data[i+1,1]-data[i,1];dD3=data[i+j+1,1]-data[i+j,1];
		dD4=data[N-i-j+1,1]-data[N-i-j,1];dD5=data[N-i+1,1]-data[N-i,1];dD6=data[N-i+j+1,1]-data[N-i+j,1];
		dDdz1=abs(dD1/dz);dDdz2=abs(dD2/dz);dDdz3=abs(dD3/dz);dDdz4=abs(dD4/dz);dDdz5=abs(dD5/dz);dDdz6=abs(dD6/dz);
		if data[i,1]> 0.3 and abs(dDdz1) < bd_s and dDdz2 < bd_s and dDdz3 < bd_s and dDdz4 < bd_s and dDdz5 < bd_s and dDdz6 < bd_s:
			print 'EDP reach plateau slope %2.7f at %2.7f ' % (dDdz3,data[i,0])
			cut_a = i; cut_b = len(data[:,0]) - cut_a
			return cut_a,cut_b

# (a,b)=plateau(data,dz,bd_s,N_wat)
# May increase bd_s 0.01 up to 0.05 to allow higher plateau fluctuation (up to 1%)
(a,b)=plateau(zDtot,0.2,0.01,N_wat)

cut_a=min(a,b); cut_b=max(a,b)
zDtot_ab=zDtot[cut_a:cut_b,:]

filetowrite = "tot_edp_cut.txt" 
np.savetxt(filetowrite,zDtot_ab,fmt='%-2.5f',delimiter='    ')
