#!/usr/bin/env python
	
# Authors:Xiaohong Zhuang, Dr.Jeffery B. Klauda
# Date:9/12/2015

# Expected No. of electrons of common atoms. (b(CH1E),m(CH2E),t(CH3E) are united atoms)
# Treat d as h, use d just to have different neutron scattering lengths defined in SIMtoEXP 
N_e={'h':1,'d':1,'c':6,'n':7,'o':8,'p':15,'s':16,'b':7,'m':8,'t':9,'k':18}

## Head group names:
# glycerolphospholip (gl)        
# LipNames:  | PA | PC | PE  | PI | PG | PS|Common Groups| 
# GrpNames:  |Acid|Chol|EolAm|Inos|GlyH|Ser|Phos|Gly|Carb|   
#     N_a :  |  1 | 19 | 10  | 22 | 12 | 12|  5 | 10|  4 |   
#     N_e :  |  1 | 50 | 26  | 87 | 41 | 47| 47 | 39| 28 | 
# Sphingolipid (sl)              
# LipNames:  |CER|   SM    |Common Grps|
# GrpNames:  |OH |Chol|Phos|Sphigo|FCO | 
#     N_a :  | 2 | 19 |  5 | 15   | 2  |  
#     N_e :  | 9 | 50 | 47 | 53   | 14 | 
# Cardiolipin (dl)              
# LipNames:  |  CL1 |  CL2 |  Common Groups |
# GrpNames:  |Phosd1|Phosd2|GlyHd|Glyd|Carbd|   
#     N_a :  |  6   |   5  |  10 | 20 |  8  | 
#     N_e :  |  48  |  47  |  32 | 78 | 56  |
# Sterols (cl)              
# LipNames:  CHL1/ERG/SITO/STIG
# GrpNames:  |OHc|    
#     N_a :  | 2 |   
#     N_e :  | 9 | 

## Tail group names:
#  Cyc,C,CH,CH2,CH3,OH, CH1E,CH2E,CH3E
## Other group names:
# Pot, Wat 

# Expected No. of atoms and electrons per group of common head groups (OH is taken care outside dictionary)
N_aeh={'Acid':[1,1],'Chol':[19,50],'EolAm':[10,26],'Inos':[22,87],'GlyH':[12,41],'Ser':[12,47],
	'Phos':[5,47],'Gly':[10,39],'Carb':[4,28],'Sphig':[15,53],'FCO':[2,14],
	'GlyHd':[10,32],'Phosd1':[6,48],'Phosd2':[5,47],'Glyd':[20,78],'Carbd':[8,56],
	'Wat':[3,10],'Pot':[1,18],'OHc':[2,9]}
	
def Expect(Ngrp,gnames,exact_a):	
	Exp_Na=[]; Exp_Ne=[]
	for i in range(0,Ngrp,1):
		# For head groups, read data from dictionary N_aeh above	
		if N_aeh.has_key(gnames[i]): 
			Exp_Na.append(N_aeh[gnames[i]][0]); Exp_Ne.append(N_aeh[gnames[i]][1])
		# For tail groups, calculate base on the No. of atoms on the 2nd column of cmp
		else: 
			if gnames[i].lower()=='Cyc'.lower():
				Exp_Na.append(exact_a[i]); Exp_Ne.append(int(exact_a[i]/7*22));
			if gnames[i].lower()=='C'.lower():
				Exp_Na.append(exact_a[i]); Exp_Ne.append(int(exact_a[i]/1*6));		
			if gnames[i].lower()=='CH'.lower():
				Exp_Na.append(exact_a[i]); Exp_Ne.append(int(exact_a[i]/2*7));			
			if gnames[i].lower()=='CH2'.lower():
				Exp_Na.append(exact_a[i]); Exp_Ne.append(int(exact_a[i]/3*8));	
			if gnames[i].lower()=='CH3'.lower():
				Exp_Na.append(exact_a[i]); Exp_Ne.append(int(exact_a[i]/4*9));			
			if gnames[i].lower()=='OH'.lower():
				Exp_Na.append(exact_a[i]); Exp_Ne.append(int(exact_a[i]/2*9));
			# United atom groups
			if gnames[i].lower()=='CH1E'.lower():
				Exp_Na.append(exact_a[i]); Exp_Ne.append(int(exact_a[i]/1*7));			
			if gnames[i].lower()=='CH2E'.lower():
				Exp_Na.append(exact_a[i]); Exp_Ne.append(int(exact_a[i]/1*8));	
			if gnames[i].lower()=='CH3E'.lower():
				Exp_Na.append(exact_a[i]); Exp_Ne.append(int(exact_a[i]/1*9));
	return Exp_Na,Exp_Ne

	