#!/usr/bin/env python
	
# Authors:Xiaohong Zhuang, Dr.Jeffery B. Klauda
# Date:9/12/2015
	
# This script works for multi-type of lipid bilayer, check the followings:
# 1. Total No. of atoms in *cmp and *sim  
# 2. Atom names mismatch between *cmp and *sim 
# 3. Zero density of an atom in *sim
# 4. No. of atoms of each group in *sim
	
import numpy as np
def check_Na_Ne(lip,N_lip,sa_tot):	
        
	## Reading and createing the files
	# Read the group names based on cmp
	filetoread = "%s-nh-grp-names.txt" % lip
	fob=open(filetoread,'r')
	data=fob.readlines()
	gnames = []
	for line in data:
		gnames.append(line)                       # To generate lists of atom names of all lines, with each line as a string element
	gnames = map(lambda s: s.strip(), gnames)        # To remove \n in each string element
	fob.close()
	
	
	# Read atom names in cmp and calculate 
	filetoread = "%s-nhc.cmp" % lip
	Ngrp = sum(1 for line in open(filetoread))
	fob=open(filetoread,'r')
	cmp_file=fob.readlines()
	atmn_cmp1 = map(lambda s: s.strip(), cmp_file) 
	# Make each string element into list of elements, which make it total list of element list
	for index, line in enumerate(atmn_cmp1):
	    atmn_cmp1[index] = line.split(' ')
	# Take the elments of list out to the main list
	atmn_cmp=[]
	for item in atmn_cmp1:	
		atmn_cmp=atmn_cmp+item
	
	# Read *.cmp for No. of atoms in each group
	filetoread = "%s-nh-N_per_grp.txt" % lip
	exact_a=np.genfromtxt(filetoread,delimiter="\n")
	
	
	# Read and generate index based on atom names in *sim
	filetoread = "%s-atms.txt" % lip
	fob=open(filetoread,'r')
	atmn_sim=fob.readline()
	atmn_sim=atmn_sim.split()
	fob.close()
	
	
	# Read *.sim file
	filetoread = "%s-nh2.sim" % lip
	sim=np.genfromtxt(filetoread,delimiter=" ")
	
	# Print lipid name and surface area to visualize the data
	print 'Multiple-type lipid Bilayer System'
	print 'lipid name: ', lip, "      ", 'Total SA_avg: ', sa_tot, "      ",'No. of groups: ', Ngrp
	print " "
	
	
	# 1. Check the total # of atoms in *cmp based on *sim
	print "PART 1. Total No. of atom names:" 
	print 'Expc.\tCMP\tSIM'
	print '%i\t%i\t%i' % (np.array(exact_a).sum(),len(atmn_cmp),len(atmn_sim))
	print " "
	
	
	## 2. Check the mismatch of atom names betweem *sim and *cmp
	print "PART 2."
	# Check the atom names in sim exist in cmp
	for i in range(0,len(atmn_sim),1):
		a_sim=atmn_sim[i]
		if a_sim in atmn_cmp:
			pass
		else:
			print 'ERROR: %s exit in the %s.sim does not exist in the %s.cmp' % (a_sim,lip,lip)
			print " "
	
	# Check the atom names in cmp exist in sim
	for i in range(0,len(atmn_cmp),1):
		a_cmp=atmn_cmp[i]
		if a_cmp in atmn_sim:
			pass
		else:
			print 'ERROR: %s exit in the %s.cmp does not exist in the %s.sim' % (a_cmp,lip,lip)
			print " "
	print 'The atom names mismatch checking between %s.sim and %s.cmp is complete.' % (lip,lip)
	print " "
	
	
	## 3. Check zero density of an atom in sim
	print "PART 3."
	if lip=='pot':
		ColSum=sum(sim[:])
		if ColSum < 0.01:
	             print "ERROR: The atom name '%s' in %s.sim has zero atomic density!" % ('k',lip)
	if lip!='pot':
		for i in range(0,len(atmn_sim),1): 
			ColSum=np.sum(sim[:,i])     # if use np.sum(array, axis=0,1) axis=0 for column sum, and axis=1 for row sum
			if ColSum < 0.01:
				print "ERROR: The %ith atom, atom name '%s' in %s.sim has zero atomic density!" % (i,atmn_sim[i],lip)
				print " "
	print 'The zero atomic density checking in %s.sim is complete.' % lip
	print " "
	

	## 4.Compare # of atoms and electrons in each group
	print "PART 4."
	
	# Import the expected data from E_data.py
	from E_data import N_e
	from E_data import N_aeh
	from E_data import Expect
	Exp_Na=Expect(Ngrp,gnames,exact_a)[0]
	Exp_Ne=Expect(Ngrp,gnames,exact_a)[1]
	
	# Calculate # of atoms and electrons in each group
	lineInd=0;  
	grpNa_cmp=[]; grpNe_cmp=[]; grpNa_sim=[]; grpNe_sim=[];
	for line in cmp_file:
		group = line.strip().split()                          		  # To generate a list atoms names of each single line called group
		Da = 0; N_e_a=0; gDa  = 0; gNe_cmp=0; gNe_sim=0;
		for i in range(0,len(group),1):
			id_atom_sim = atmn_sim.index(group[i])               	  # To find the index of the cmp atom names in atms.txt list
			if lip=='pot':
				Da = sum(sim[:])			
			if lip!='pot':			
				Da = np.array(sim[:,id_atom_sim]).sum();
			gDa = gDa + Da;
			letters = map(lambda s: s.strip(), group[i])
			N_e_a=N_e[letters[0]]
			gNe_cmp=gNe_cmp+N_e_a
			gNe_sim=gNe_sim+Da*N_e[letters[0]]
		gNa_sim = gDa *0.2* sa_tot / (2.0 * N_lip)                       # Formular: gNa_sim = (Atom density)*(binsize 0.2)* (total SA)/(2 * N_lip)
		gNe_sim = gNe_sim*0.2* sa_tot / (2.0 * N_lip)
		gNa_cmp = len(group)
		grpNa_cmp.append(gNa_cmp);grpNe_cmp.append(gNe_cmp)
		grpNa_sim.append(gNa_sim); grpNe_sim.append(gNe_sim);
		if Ngrp>1: pass
		else: exact_a=[exact_a.tolist()];                                # Convert array of one element to list
		fracdev = abs(gNa_sim - exact_a[lineInd])/exact_a[lineInd]
		if fracdev > 0.01:                                               # Print alert when fraction deviation is greater than 0.01
			print 'Attention! The Deviation fraction of e- No. for %s %1.3f exceeds 0.01 !' % (gnames[lineInd],fracdev)
		lineInd=lineInd+1
	print " "
	print '\tNo. of Atoms\t\t\t\t\tNo. of electrons'
	print 'Group\tExpc.\tCMP\tSIM\t\t\tGroup\tExpc.\tCMP\tSIM'
	if lip=='tip3' or lip=='pot':
		print '%s\t%i\t%i\t%3.2f\t\t\t%s\t%i\t%i\t%3.2f' % (gnames[0],sum(Exp_Na),sum(grpNa_cmp),sum(grpNa_sim),gnames[0],sum(Exp_Ne),sum(grpNe_cmp),sum(grpNe_sim))
	else:
		for i in range(0,Ngrp,1):
			print '%s\t%i\t%i\t%3.2f\t\t\t%s\t%i\t%i\t%3.2f' % (gnames[i],Exp_Na[i],grpNa_cmp[i],grpNa_sim[i],gnames[i],Exp_Ne[i],grpNe_cmp[i],grpNe_sim[i])
		print " "
		print 'Total:\t%i\t%i\t%3.2f\t\t\tTotal:\t%i\t%i\t%3.2f' % (sum(Exp_Na),sum(grpNa_cmp),sum(grpNa_sim),sum(Exp_Ne),sum(grpNe_cmp),sum(grpNe_sim))


	
	
